#ifndef DC_MOTOR_H_
#define DC_MOTOR_H_


#define HAL_TIM_MODULE_ENABLED

#include "stm32l4xx_hal.h"
#include <Dexter_wrappers.h>

// DC Motor Rotation Directions
#define forward    0
#define backward   1


// DC Motor PWM Properties
#define DC_MOTOR_PWM_RES  10
#define DC_MOTOR_F_PWM    500


#define Motor_R    0
#define Motor_L    1


typedef struct
{
	GPIO_TypeDef * BKWD_DIR_GPIO;
	GPIO_TypeDef * FWD_DIR_GPIO;
	uint16_t       BKWD_DIR_PIN;
	uint16_t       FWD_DIR_PIN;
	TIM_TypeDef*   TIM_Instance;
	uint32_t       PWM_TIM_CH;
	uint16_t       TIM_CLK_MHz;
	uint32_t       PWM_FREQ_Hz;
	uint8_t        PWM_RES_BITS;
}Motor_config;


/*-----[ Prototypes For All Functions ]-----*/

void Motor_init(uint8_t motor_instance);
void Motor_set_dir(uint8_t motor_instance, uint8_t dir);
void Motor_set_speed(uint8_t motor_instance, uint16_t speed);
void Motor_run(uint8_t motor_instance, uint8_t dir, uint16_t speed);
void Motor_stop(uint8_t motor_instance);
void Move_forward(uint16_t speed);
void Move_backward(uint16_t speed);
void Move(uint8_t L_motor_dir, uint16_t L_motor_speed, uint8_t R_motor_dir, uint16_t R_motor_speed);
void Turn_left_degrees(uint16_t degrees);
void Turn_right_degrees(uint16_t degrees);



#endif /* DC_MOTOR_H_ */




#ifndef DC_MOTOR_CFG_H_
#define DC_MOTOR_CFG_H_

#include <Motor.h>

extern const Motor_config Motor_params[2];

#endif /* DC_MOTOR_CFG_H_ */
