#include "Motor.h"
#include "main.h"


uint32_t ARR_Value;
uint32_t PSC_Value;

extern uint32_t turn_calibration;


const Motor_config Motor_params[2] =
{
	// Right DC MOTOR Configurations
    {
    	R_Motor_IN4_GPIO_Port,
		R_Motor_IN3_GPIO_Port,
		R_Motor_IN4_Pin,
		R_Motor_IN3_Pin,
		TIM2,
		TIM_CHANNEL_2,
		72,
		DC_MOTOR_F_PWM,
		DC_MOTOR_PWM_RES
    },

	// Left DC MOTOR Configurations
	{
		L_Motor_IN2_GPIO_Port,
		L_Motor_IN1_GPIO_Port,
		L_Motor_IN2_Pin,
		L_Motor_IN1_Pin,
		TIM2,
		TIM_CHANNEL_1,
		72,
		DC_MOTOR_F_PWM,
		DC_MOTOR_PWM_RES
	}
};



void Motor_init(uint8_t motor_instance)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};
	TIM_ClockConfigTypeDef sClockSourceConfig = {0};
    TIM_MasterConfigTypeDef sMasterConfig = {0};
    TIM_OC_InitTypeDef sConfigOC = {0};
    TIM_HandleTypeDef htim;
    ARR_Value = 0;
    PSC_Value = 0;
    uint8_t i = 0;

	/*--------[ Configure The 2 Direction Control GPIO Pins ]-------*/

    if(Motor_params[motor_instance].FWD_DIR_GPIO == GPIOA || Motor_params[motor_instance].BKWD_DIR_GPIO == GPIOA)
    {
    	__HAL_RCC_GPIOA_CLK_ENABLE();
    }
    else if(Motor_params[motor_instance].FWD_DIR_GPIO == GPIOB || Motor_params[motor_instance].BKWD_DIR_GPIO == GPIOB)
    {
    	__HAL_RCC_GPIOB_CLK_ENABLE();
    }
    else if(Motor_params[motor_instance].FWD_DIR_GPIO == GPIOC || Motor_params[motor_instance].BKWD_DIR_GPIO == GPIOC)
    {
        __HAL_RCC_GPIOC_CLK_ENABLE();
    }

	GPIO_InitStruct.Pin = Motor_params[motor_instance].FWD_DIR_PIN;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(Motor_params[motor_instance].FWD_DIR_GPIO, &GPIO_InitStruct);
	GPIO_InitStruct.Pin = Motor_params[motor_instance].BKWD_DIR_PIN;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(Motor_params[motor_instance].BKWD_DIR_GPIO, &GPIO_InitStruct);
	HAL_GPIO_WritePin(Motor_params[motor_instance].FWD_DIR_GPIO, Motor_params[motor_instance].FWD_DIR_PIN, 0);
	HAL_GPIO_WritePin(Motor_params[motor_instance].BKWD_DIR_GPIO, Motor_params[motor_instance].BKWD_DIR_PIN, 0);

	/*--------[ Calculate The PSC & ARR Values To Set PWM Resolution And Approx. The F_pwm ]-------*/

	/* Those Equations Sets The PWM Resolution & Approximates The F_pwm */
	ARR_Value = 1;
	for(i=0; i<Motor_params[motor_instance].PWM_RES_BITS; i++)
	{
		ARR_Value *= 2;
	}
	PSC_Value = (uint32_t) ((Motor_params[motor_instance].TIM_CLK_MHz*1000000) / (ARR_Value*Motor_params[motor_instance].PWM_FREQ_Hz));
	PSC_Value--;
	ARR_Value -= 2;

	/*--------[ Configure The DC Motor PWM Timer Channel ]-------*/

	htim.Instance = Motor_params[motor_instance].TIM_Instance;
	htim.Init.Prescaler = PSC_Value;
	htim.Init.CounterMode = TIM_COUNTERMODE_UP;
	htim.Init.Period = ARR_Value;
	htim.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
	htim.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_ENABLE;
	HAL_TIM_Base_Init(&htim);
	sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
	HAL_TIM_ConfigClockSource(&htim, &sClockSourceConfig);
	HAL_TIM_PWM_Init(&htim);
	sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
	sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
	HAL_TIMEx_MasterConfigSynchronization(&htim, &sMasterConfig);
	sConfigOC.OCMode = TIM_OCMODE_PWM1;
	sConfigOC.Pulse = 0;
	sConfigOC.OCPolarity = TIM_OCPOLARITY_HIGH;
	sConfigOC.OCFastMode = TIM_OCFAST_DISABLE;
	HAL_TIM_PWM_ConfigChannel(&htim, &sConfigOC, Motor_params[motor_instance].PWM_TIM_CH);
	HAL_TIM_MspPostInit(&htim);

	/*--------[ Start The PWM Channel ]-------*/

	HAL_TIM_PWM_Start(&htim, Motor_params[motor_instance].PWM_TIM_CH);
}


void Motor_set_dir(uint8_t motor_instance, uint8_t dir)
{
	/* Write To The 2 Direction Control Pins */
	if(dir == forward)
	{
		HAL_GPIO_WritePin(Motor_params[motor_instance].FWD_DIR_GPIO, Motor_params[motor_instance].FWD_DIR_PIN, 1);
		HAL_GPIO_WritePin(Motor_params[motor_instance].BKWD_DIR_GPIO, Motor_params[motor_instance].BKWD_DIR_PIN, 0);
	}
	else if(dir == backward)
	{
		HAL_GPIO_WritePin(Motor_params[motor_instance].FWD_DIR_GPIO, Motor_params[motor_instance].FWD_DIR_PIN, 0);
		HAL_GPIO_WritePin(Motor_params[motor_instance].BKWD_DIR_GPIO, Motor_params[motor_instance].BKWD_DIR_PIN, 1);
	}
}


void Motor_set_speed(uint8_t motor_instance, uint16_t speed)
{
	/* Write The Speed Value To The PWM CH DutyCycle Register */
	if(Motor_params[motor_instance].PWM_TIM_CH == TIM_CHANNEL_1)
	{
		Motor_params[motor_instance].TIM_Instance->CCR1 = speed*30;
	}
	else if(Motor_params[motor_instance].PWM_TIM_CH == TIM_CHANNEL_2)
	{
		Motor_params[motor_instance].TIM_Instance->CCR2 = speed*30;
	}
	else if(Motor_params[motor_instance].PWM_TIM_CH == TIM_CHANNEL_3)
	{
		Motor_params[motor_instance].TIM_Instance->CCR3 = speed*30;
	}
	else
	{
		Motor_params[motor_instance].TIM_Instance->CCR4 = speed*30;
	}
}


void Motor_run(uint8_t motor_instance, uint8_t dir, uint16_t speed)
{
	Motor_set_dir(motor_instance, dir);
	Motor_set_speed(motor_instance, speed);
}

void Motor_stop(uint8_t motor_instance)
{
	HAL_GPIO_WritePin(Motor_params[motor_instance].FWD_DIR_GPIO, Motor_params[motor_instance].FWD_DIR_PIN, 0);
	HAL_GPIO_WritePin(Motor_params[motor_instance].BKWD_DIR_GPIO, Motor_params[motor_instance].BKWD_DIR_PIN, 0);

	Motor_set_speed(motor_instance, 0);
}









void Move_forward(uint16_t speed)
{
	Motor_run(Motor_L, forward, speed);
	Motor_run(Motor_R, forward, speed);
}

void Move_backward(uint16_t speed)
{
	Motor_run(Motor_L, backward, speed);
	Motor_run(Motor_R, backward, speed);
}

void Move(uint8_t L_motor_dir, uint16_t L_motor_speed, uint8_t R_motor_dir, uint16_t R_motor_speed)
{
	Motor_run(Motor_L, L_motor_dir, L_motor_speed);
	Motor_run(Motor_R, R_motor_dir, R_motor_speed);
}

void Turn_left_degrees(uint16_t degrees)
{
	uint16_t duration = (degrees/90)*turn_calibration;

	Motor_run(Motor_L, backward, 50);
	Motor_run(Motor_R, forward, 50);
	Delay_ms(duration);
	Motor_stop(Motor_L);
	Motor_stop(Motor_R);
}

void Turn_right_degrees(uint16_t degrees)
{
	uint16_t duration = (degrees/90)*turn_calibration;

	Motor_run(Motor_L, forward, 50);
	Motor_run(Motor_R, backward, 50);
	Delay_ms(duration);
	Motor_stop(Motor_L);
	Motor_stop(Motor_R);
}
