#include "Ultrasonic.h"
#include "main.h"


#define L_US_TRIG_PORT	'B'
#define L_US_TRIG_PIN	 9
#define L_US_ECHO_PORT	'B'
#define L_US_ECHO_PIN	 15

#define F_US_TRIG_PORT	'A'
#define F_US_TRIG_PIN	 10
#define F_US_ECHO_PORT	'A'
#define F_US_ECHO_PIN	 9

#define R_US_TRIG_PORT	'B'
#define R_US_TRIG_PIN	 14
#define R_US_ECHO_PORT	'A'
#define R_US_ECHO_PIN	 8

uint32_t numTicks = 0;
uint32_t Sensor_one_ticks = 0;
uint32_t Sensor_two_ticks = 0;

extern uint16_t ultrasonic_calibration;

void Delay_us(uint32_t uSec)
{
	if(uSec < 2) uSec = 2;
	TIM1->ARR = uSec - 1; 	/*sets the value in the auto-reload register*/
	TIM1->EGR = 1; 			/*Re-initialises the timer*/
	TIM1->SR &= ~1; 		//Resets the flag
	TIM1->CR1 |= 1; 		//Enables the counter
	while((TIM1->SR&0x0001) != 1);
	TIM1->SR &= ~(0x0001);
}


float Read_L_ultrasonic()
{
	// Set TRIG pin LOW for few microseconds
	digital_write(L_US_TRIG_PORT, L_US_TRIG_PIN, 0);
	Delay_us(2);

	//*** START Ultrasonic measure routine ***//

	// 1) Set TRIG pin HIGH for 10 microseconds
	digital_write(L_US_TRIG_PORT, L_US_TRIG_PIN, 1);
	Delay_us(10);
	digital_write(L_US_TRIG_PORT, L_US_TRIG_PIN, 0);

	// 2) Wait for ECHO pin rising edge
	while(digital_read(L_US_ECHO_PORT, L_US_ECHO_PIN) == 0);

	// 3) Start measuring ECHO pulse width in microseconds
	float L_ultrasonic_ticks = 0;
	while(digital_read(L_US_ECHO_PORT, L_US_ECHO_PIN) == 1)
	{
		L_ultrasonic_ticks++;
		Delay_us(2);
	};

	Delay_ms(60);

	// 4) Estimate distance in cm and return
	return (L_ultrasonic_ticks*2)/ultrasonic_calibration;
}


float Read_F_ultrasonic()
{


	//Set TRIG to LOW for few uSec -  second sensor
	digital_write(F_US_TRIG_PORT, F_US_TRIG_PIN, 0);
	Delay_us(2);

	//*** START Ultrasonic measure routine ***//
	//1. Output 10 usec TRIG
	digital_write(F_US_TRIG_PORT, F_US_TRIG_PIN, 1);
	Delay_us(10);
	digital_write(F_US_TRIG_PORT, F_US_TRIG_PIN, 0);

	//2. Wait for ECHO pin rising edge
	while(digital_read(F_US_ECHO_PORT, F_US_ECHO_PIN) == 0);

	//3. Start measuring ECHO pulse width in usec
	float F_ultrasonic_ticks = 0;
	while(digital_read(F_US_ECHO_PORT, F_US_ECHO_PIN) == 1)
	{
		F_ultrasonic_ticks++;
		Delay_us(2); //2.8usec
	};

	Delay_ms(60);

	//4. Estimate distance in cm
	//  + 0.0f)*2.8*speedOfSound;

	return (F_ultrasonic_ticks*2)/ultrasonic_calibration;
}


float Read_R_ultrasonic()
{

	//Set TRIG to LOW for few uSec - first sensor
	digital_write(R_US_TRIG_PORT, R_US_TRIG_PIN, 0);
	Delay_us(2);

	//*** START Ultrasonic measure routine ***//
	//1. Output 10 usec TRIG
	digital_write(R_US_TRIG_PORT, R_US_TRIG_PIN, 1);
	Delay_us(10);
	digital_write(R_US_TRIG_PORT, R_US_TRIG_PIN, 0);

	//2. Wait for ECHO pin rising edge
	while(digital_read(R_US_ECHO_PORT, R_US_ECHO_PIN) == 0);

	//3. Start measuring ECHO pulse width in usec
	float R_ultrasonic_ticks = 0;
	while(digital_read(R_US_ECHO_PORT, R_US_ECHO_PIN) == 1)
	{
		R_ultrasonic_ticks++;
		Delay_us(2); //2.8usec
	};

	Delay_ms(60);

	//4. Estimate distance in cm
	//  + 0.0f)*2.8*speedOfSound;

	return (R_ultrasonic_ticks*2)/ultrasonic_calibration;
}
